@@ -1,5 +1,3 @@
-from gtfstk import read_gtfs, Feed
-from pandas import merge, DataFrame
 from pathlib import Path
 from typing import (
     Any,
@@ -10,6 +8,12 @@ from typing import (
     Tuple,
     Union,
 )
+from gtfs_kit import Feed
+from gtfs_kit import read_feed as read_gtfs
+from pandas import (
+    DataFrame,
+    merge,
+)
 from .utils import log_progress
 
 
@@ -32,14 +36,14 @@ def extract_data_from_feed(feed: Feed) -> Tuple[DataFrame]:
     routes = feed.routes
     trips = feed.trips
     stop_times = feed.stop_times
-    calendar = feed.calendar if feed.calendar is not None else None
-    calendar_dates = feed.calendar_dates if feed.calendar_dates is not None else None
-    fare_attributes = feed.fare_attributes if feed.fare_attributes is not None else None
-    fare_rules = feed.fare_rules if feed.fare_rules is not None else None
-    shapes = feed.shapes if feed.shapes is not None else None
-    frequencies = feed.frequencies if feed.frequencies is not None else None
-    transfers = feed.transfers if feed.transfers is not None else None
-    feed_info = feed.feed_info if feed.feed_info is not None else None
+    calendar = feed.calendar
+    calendar_dates = feed.calendar_dates
+    fare_attributes = feed.fare_attributes
+    fare_rules = feed.fare_rules
+    shapes = feed.shapes
+    frequencies = feed.frequencies
+    transfers = feed.transfers
+    feed_info = feed.feed_info
     return (agency, stops, routes, trips, stop_times,
             calendar, calendar_dates, fare_attributes, fare_rules,
             shapes, frequencies, transfers, feed_info)
@@ -64,9 +68,11 @@ def add_suffix(
     """
     if data is not None and len(to_suffix):
         entries_to_suffix = data.loc[:, column_name].isin(to_suffix)
-        result = data
+        result = data.copy()
         result.loc[entries_to_suffix, column_name] = data.loc[:, column_name] + suffix
-    return result
+        return result
+    else:
+        return data
 
 
 def append(df1: DataFrame, df2: DataFrame, column_name: str, suffix: str) -> Tuple[DataFrame, Set[Any]]:
@@ -104,12 +110,9 @@ def add_data(data: DataFrame, data_to_add: DataFrame) -> DataFrame:
         the aggregated dataframe
     """
     if data_to_add is None:
-        result_data = data
+        result_data = None if data is None else data.copy()
     else:
-        if data is None:
-            result_data = data_to_add
-        else:
-            result_data = data.append(data_to_add)
+        result_data = data_to_add if data is None else data.append(data_to_add)
     return result_data
 
 
@@ -126,9 +129,9 @@ def append_and_suffix(data: DataFrame, data_to_add: DataFrame, column_ref: str,
     Returns:
         result of concatenation
     """
-    agency, to_suffix = append(data, data_to_add, column_ref, suffix)
-    for data_to_add, column in data_to_suffix:
-        add_suffix(data_to_add, column, to_suffix, suffix)
+    data, to_suffix = append(data, data_to_add, column_ref, suffix)
+    for df_to_suffix, column in data_to_suffix:
+        add_suffix(df_to_suffix, column, to_suffix, suffix)
     return data
 
 
@@ -153,6 +156,7 @@ def merge_gtfs(origin_gtfs_path: Union[Path, str]) -> Feed:
         return None
     # Get dataframes of the first feed
     first_feed = read_gtfs(paths[0], dist_units='km')
+    log_progress(f"Merging file 1/{files_count}: {paths[0]}")
     (
         agency, stops, routes, trips, stop_times,
         calendar, calendar_dates, fare_attributes, fare_rules,
@@ -168,24 +172,50 @@ def merge_gtfs(origin_gtfs_path: Union[Path, str]) -> Feed:
             calendar_to_add, calendar_dates_to_add, fare_attributes_to_add, fare_rules_to_add,
             shapes_to_add, frequencies_to_add, transfers_to_add, feed_info_to_add
         ) = extract_data_from_feed(feed=feed_to_add)
-        append_and_suffix(data=agency, data_to_add=agency_to_add, column_ref='agency_id', suffix=suffix,
-                          data_to_suffix=[(routes_to_add, 'agency_id'), (fare_attributes_to_add, 'agency_id')])
-        append_and_suffix(data=stops, data_to_add=stops_to_add, column_ref='stop_id', suffix=suffix,
-                          data_to_suffix=[(stop_times_to_add, 'stop_id'), (transfers_to_add, 'from_stop_id'),
-                                          (transfers_to_add, 'to_stop_id')])
-        append_and_suffix(data=routes, data_to_add=routes_to_add, column_ref='route_id', suffix=suffix,
-                          data_to_suffix=[(trips_to_add, 'route_id'), (fare_rules_to_add, 'route_id')])
-        append_and_suffix(data=trips, data_to_add=trips_to_add, column_ref='trip_id', suffix=suffix,
-                          data_to_suffix=[(stop_times_to_add, 'trip_id'), (frequencies_to_add, 'trip_id')])
-        stop_times = append(stop_times, stop_times_to_add)  # no suffix needed
-        append_and_suffix(data=calendar, data_to_add=calendar_dates_to_add, column_ref='service_id', suffix=suffix,
-                          data_to_suffix=[(trips_to_add, 'service_id'), (calendar_dates_to_add, 'service_id')])
-        calendar_dates = add_data(calendar_dates, calendar_dates_to_add)  # no suffix needed
-        append_and_suffix(data=fare_attributes, data_to_add=fare_attributes_to_add, column_ref='fare_id', suffix=suffix,
-                          data_to_suffix=[(fare_rules_to_add, 'fare_id')])
-        fare_rules = append(fare_rules, fare_rules_to_add, 'fare_id', suffix)  # no suffix needed
-        append_and_suffix(data=shapes, data_to_add=shapes_to_add, column_ref='shape_id', suffix=suffix,
-                          data_to_suffix=[(trips_to_add, 'shape_id')])
+        agency = append_and_suffix(data=agency, data_to_add=agency_to_add,
+                                   column_ref='agency_id', suffix=suffix,
+                                   data_to_suffix=[
+                                       (routes_to_add, 'agency_id'),
+                                       (fare_attributes_to_add, 'agency_id')
+                                   ])
+        stops = append_and_suffix(data=stops, data_to_add=stops_to_add,
+                                  column_ref='stop_id', suffix=suffix,
+                                  data_to_suffix=[
+                                      (stop_times_to_add, 'stop_id'),
+                                      (transfers_to_add, 'from_stop_id'),
+                                      (transfers_to_add, 'to_stop_id')
+                                  ])
+        routes = append_and_suffix(data=routes, data_to_add=routes_to_add,
+                                   column_ref='route_id', suffix=suffix,
+                                   data_to_suffix=[
+                                       (trips_to_add, 'route_id'),
+                                       (fare_rules_to_add, 'route_id')
+                                   ])
+        trips = append_and_suffix(data=trips, data_to_add=trips_to_add,
+                                  column_ref='trip_id', suffix=suffix,
+                                  data_to_suffix=[
+                                      (stop_times_to_add, 'trip_id'),
+                                      (frequencies_to_add, 'trip_id')
+                                  ])
+        stop_times = add_data(stop_times, stop_times_to_add)
+        calendar = append_and_suffix(data=calendar, data_to_add=calendar_dates_to_add,
+                                     column_ref='service_id', suffix=suffix,
+                                     data_to_suffix=[
+                                         (trips_to_add, 'service_id'),
+                                         (calendar_dates_to_add, 'service_id')
+                                     ])
+        calendar_dates = add_data(calendar_dates, calendar_dates_to_add)
+        fare_attributes = append_and_suffix(data=fare_attributes, data_to_add=fare_attributes_to_add,
+                                            column_ref='fare_id', suffix=suffix,
+                                            data_to_suffix=[
+                                                (fare_rules_to_add, 'fare_id')
+                                            ])
+        fare_rules = add_data(fare_rules, fare_rules_to_add)
+        shapes = append_and_suffix(data=shapes, data_to_add=shapes_to_add,
+                                   column_ref='shape_id', suffix=suffix,
+                                   data_to_suffix=[
+                                       (trips_to_add, 'shape_id')
+                                   ])
         frequencies = add_data(frequencies, frequencies_to_add)
         transfers = add_data(transfers, transfers_to_add)
         feed_info = add_data(feed_info, feed_info_to_add)
@@ -364,9 +394,10 @@ def remove_routes_from_feed_by_short_names(feed: Feed, routes_to_remove_short_na
 
 
 def group_routes(feed: Feed) -> Feed:
-    """Regroup routes when thay have the same short name and the same agency.
+    """Regroup routes when they have the same short name and the same agency.
 
     Resolve the problem of patterns dispached in several routes.
+    Add the agency id as a prefix to route short name.
 
     Args:
         feed: feed to correct
