@@ -100,7 +100,7 @@ def calculate_density(
         field_to = field_from + '_dens'
     if area_field:
         unit_adjustment = 1 if area_field_unit_in_km2 else 1e6
-        result[field_to] = data[field_from] / data[area_field] / unit_adjustment
+        result[field_to] = data[field_from] / data[area_field] * unit_adjustment
     else:
         # geopandas' area gives a value in m²
         result[field_to] = data[field_from] / data.area * 1e6
@@ -217,12 +217,11 @@ def clean_layer(
 def create_carroyage(
     carroyage_path: Union[str, Path],
     data_path: Union[str, Path],
-    output_path: Union[str, Path],
-    bbox: List[List[float]] = None,
-    min_lat: float = None,
-    max_lat: float = None,
-    min_lon: float = None,
-    max_lon: float = None
+    bbox: Optional[List[List[float]]] = None,
+    min_lat: Optional[float] = None,
+    max_lat: Optional[float] = None,
+    min_lon: Optional[float] = None,
+    max_lon: Optional[float] = None,
 ) -> None:
     """Process the carroyage, cuts it to the necessary bounding box and export the result into a shapefile.
 
@@ -240,7 +239,7 @@ def create_carroyage(
     population_data = read_population_data(data_path)
     carroyage = carroyage.merge(population_data, on='id')
     # selecting the carroyage in bbox_wgs84
-    if min_lat or max_lat or min_lon or max_lon:
+    if min_lat or max_lat or min_lon or max_lon or bbox:
         proj_carroyage = Proj(carroyage.crs)
         proj_bbox = Proj(init='epsg:4326')
         # setting defaults
@@ -257,7 +256,7 @@ def create_carroyage(
         carroyage = carroyage.cx[min_lon:max_lon, min_lat:max_lat]
     carroyage['pop_dens'] = carroyage['ind_c'] / (200 * 200 / 1e6)  # by definition, tiles are 200m wide
     carroyage = carroyage.to_crs(epsg=4326)
-    carroyage.to_file(output_path)
+    return carroyage
 
 
 def read_population_data(path: Union[str, Path]) -> GeoDataFrame:
@@ -340,7 +339,6 @@ def merge_and_clean_layers(
     """Create a layer from several shapefiles and clean it by only keeping columns of given names.
 
     The given shapefiles should be of the same format (same columns) as much as possible.
-    It also projects the generated layer in WGS84.
 
     Args:
         paths: file paths to merge
